@@ -11,6 +11,7 @@ use serde::ser;
 use serde::Serialize;
 use url::Url;
 
+use crate::core::compiler::rustdoc::RustdocScrapeExamples;
 use crate::core::compiler::{CompileKind, CrateType};
 use crate::core::resolver::ResolveBehavior;
 use crate::core::{Dependency, PackageId, PackageIdSpec, SourceId, Summary};
@@ -25,6 +26,15 @@ pub enum EitherManifest {
     Virtual(VirtualManifest),
 }
 
+impl EitherManifest {
+    pub(crate) fn workspace_config(&self) -> &WorkspaceConfig {
+        match *self {
+            EitherManifest::Real(ref r) => r.workspace_config(),
+            EitherManifest::Virtual(ref v) => v.workspace_config(),
+        }
+    }
+}
+
 /// Contains all the information about a package, as loaded from a `Cargo.toml`.
 ///
 /// This is deserialized using the [`TomlManifest`] type.
@@ -210,6 +220,7 @@ struct TargetInner {
     for_host: bool,
     proc_macro: bool,
     edition: Edition,
+    doc_scrape_examples: RustdocScrapeExamples,
 }
 
 #[derive(Clone, PartialEq, Eq, PartialOrd, Ord)]
@@ -259,7 +270,7 @@ struct SerializedTarget<'a> {
     /// Serialized as a list of strings for historical reasons.
     kind: &'a TargetKind,
     /// Corresponds to `--crate-type` compiler attribute.
-    /// See https://doc.rust-lang.org/reference/linkage.html
+    /// See <https://doc.rust-lang.org/reference/linkage.html>
     crate_types: Vec<CrateType>,
     name: &'a str,
     src_path: Option<&'a PathBuf>,
@@ -267,7 +278,7 @@ struct SerializedTarget<'a> {
     #[serde(rename = "required-features", skip_serializing_if = "Option::is_none")]
     required_features: Option<Vec<&'a str>>,
     /// Whether docs should be built for the target via `cargo doc`
-    /// See https://doc.rust-lang.org/cargo/commands/cargo-doc.html#target-selection
+    /// See <https://doc.rust-lang.org/cargo/commands/cargo-doc.html#target-selection>
     doc: bool,
     doctest: bool,
     /// Whether tests should be run for the target (`test` field in `Cargo.toml`)
@@ -363,6 +374,7 @@ compact_debug! {
                 for_host
                 proc_macro
                 edition
+                doc_scrape_examples
             )]
         }
     }
@@ -638,6 +650,7 @@ impl Target {
                 harness: true,
                 for_host: false,
                 proc_macro: false,
+                doc_scrape_examples: RustdocScrapeExamples::Unset,
                 edition,
                 tested: true,
                 benched: true,
@@ -689,7 +702,8 @@ impl Target {
             .set_name(name)
             .set_for_host(true)
             .set_benched(false)
-            .set_tested(false);
+            .set_tested(false)
+            .set_doc_scrape_examples(RustdocScrapeExamples::Disabled);
         target
     }
 
@@ -700,7 +714,8 @@ impl Target {
             .set_name(name)
             .set_for_host(true)
             .set_benched(false)
-            .set_tested(false);
+            .set_tested(false)
+            .set_doc_scrape_examples(RustdocScrapeExamples::Disabled);
         target
     }
 
@@ -794,6 +809,9 @@ impl Target {
     pub fn edition(&self) -> Edition {
         self.inner.edition
     }
+    pub fn doc_scrape_examples(&self) -> RustdocScrapeExamples {
+        self.inner.doc_scrape_examples
+    }
     pub fn benched(&self) -> bool {
         self.inner.benched
     }
@@ -828,6 +846,13 @@ impl Target {
         }
     }
 
+    pub fn is_staticlib(&self) -> bool {
+        match self.kind() {
+            TargetKind::Lib(libs) => libs.iter().any(|l| *l == CrateType::Staticlib),
+            _ => false,
+        }
+    }
+
     /// Returns whether this target produces an artifact which can be linked
     /// into a Rust crate.
     ///
@@ -901,6 +926,13 @@ impl Target {
         Arc::make_mut(&mut self.inner).edition = edition;
         self
     }
+    pub fn set_doc_scrape_examples(
+        &mut self,
+        doc_scrape_examples: RustdocScrapeExamples,
+    ) -> &mut Target {
+        Arc::make_mut(&mut self.inner).doc_scrape_examples = doc_scrape_examples;
+        self
+    }
     pub fn set_harness(&mut self, harness: bool) -> &mut Target {
         Arc::make_mut(&mut self.inner).harness = harness;
         self
