@@ -325,8 +325,8 @@ fn add_pkg(
     let node_features = resolved_features.activated_features(package_id, features_for);
     let node_kind = match features_for {
         FeaturesFor::HostDep => CompileKind::Host,
-        FeaturesFor::NormalOrDevOrArtifactTarget(Some(target)) => CompileKind::Target(target),
-        FeaturesFor::NormalOrDevOrArtifactTarget(None) => requested_kind,
+        FeaturesFor::ArtifactDep(target) => CompileKind::Target(target),
+        FeaturesFor::NormalOrDev => requested_kind,
     };
     let node = Node::Package {
         package_id,
