@@ -1,8 +1,7 @@
-use crate::core::compiler::{BuildConfig, MessageFormat};
+use crate::core::compiler::{BuildConfig, MessageFormat, TimingOutput};
 use crate::core::resolver::CliFeatures;
 use crate::core::{Edition, Workspace};
 use crate::ops::{CompileFilter, CompileOptions, NewOptions, Packages, VersionControl};
-use crate::sources::CRATES_IO_REGISTRY;
 use crate::util::important_paths::find_root_manifest_for_wd;
 use crate::util::interning::InternedString;
 use crate::util::restricted_names::is_glob_pattern;
@@ -15,18 +14,17 @@ use crate::util::{
 use crate::CargoResult;
 use anyhow::bail;
 use cargo_util::paths;
-use clap::{self, SubCommand};
 use std::ffi::{OsStr, OsString};
 use std::path::PathBuf;
 
 pub use crate::core::compiler::CompileMode;
 pub use crate::{CliError, CliResult, Config};
-pub use clap::{AppSettings, Arg, ArgMatches};
+pub use clap::{value_parser, Arg, ArgAction, ArgMatches};
 
-pub type App = clap::App<'static, 'static>;
+pub use clap::Command;
 
-pub trait AppExt: Sized {
-    fn _arg(self, arg: Arg<'static, 'static>) -> Self;
+pub trait CommandExt: Sized {
+    fn _arg(self, arg: Arg) -> Self;
 
     /// Do not use this method, it is only for backwards compatibility.
     /// Use `arg_package_spec_no_all` instead.
@@ -37,7 +35,7 @@ pub trait AppExt: Sized {
         exclude: &'static str,
     ) -> Self {
         self.arg_package_spec_no_all(package, all, exclude)
-            ._arg(opt("all", "Alias for --workspace (deprecated)"))
+            ._arg(flag("all", "Alias for --workspace (deprecated)"))
     }
 
     /// Variant of arg_package_spec that does not include the `--all` flag
@@ -50,18 +48,18 @@ pub trait AppExt: Sized {
         exclude: &'static str,
     ) -> Self {
         self.arg_package_spec_simple(package)
-            ._arg(opt("workspace", all))
+            ._arg(flag("workspace", all))
             ._arg(multi_opt("exclude", "SPEC", exclude))
     }
 
     fn arg_package_spec_simple(self, package: &'static str) -> Self {
-        self._arg(optional_multi_opt("package", "SPEC", package).short("p"))
+        self._arg(optional_multi_opt("package", "SPEC", package).short('p'))
     }
 
     fn arg_package(self, package: &'static str) -> Self {
         self._arg(
             optional_opt("package", package)
-                .short("p")
+                .short('p')
                 .value_name("SPEC"),
         )
     }
@@ -69,9 +67,14 @@ pub trait AppExt: Sized {
     fn arg_jobs(self) -> Self {
         self._arg(
             opt("jobs", "Number of parallel jobs, defaults to # of CPUs")
-                .short("j")
-                .value_name("N"),
+                .short('j')
+                .value_name("N")
+                .allow_hyphen_values(true),
         )
+        ._arg(flag(
+            "keep-going",
+            "Do not abort the build as soon as there is an error (unstable)",
+        ))
     }
 
     fn arg_targets_all(
@@ -89,10 +92,10 @@ pub trait AppExt: Sized {
     ) -> Self {
         self.arg_targets_lib_bin_example(lib, bin, bins, example, examples)
             ._arg(optional_multi_opt("test", "NAME", test))
-            ._arg(opt("tests", tests))
+            ._arg(flag("tests", tests))
             ._arg(optional_multi_opt("bench", "NAME", bench))
-            ._arg(opt("benches", benches))
-            ._arg(opt("all-targets", all))
+            ._arg(flag("benches", benches))
+            ._arg(flag("all-targets", all))
     }
 
     fn arg_targets_lib_bin_example(
@@ -103,11 +106,11 @@ pub trait AppExt: Sized {
         example: &'static str,
         examples: &'static str,
     ) -> Self {
-        self._arg(opt("lib", lib))
+        self._arg(flag("lib", lib))
             ._arg(optional_multi_opt("bin", "NAME", bin))
-            ._arg(opt("bins", bins))
+            ._arg(flag("bins", bins))
             ._arg(optional_multi_opt("example", "NAME", example))
-            ._arg(opt("examples", examples))
+            ._arg(flag("examples", examples))
     }
 
     fn arg_targets_bins_examples(
@@ -118,9 +121,9 @@ pub trait AppExt: Sized {
         examples: &'static str,
     ) -> Self {
         self._arg(optional_multi_opt("bin", "NAME", bin))
-            ._arg(opt("bins", bins))
+            ._arg(flag("bins", bins))
             ._arg(optional_multi_opt("example", "NAME", example))
-            ._arg(opt("examples", examples))
+            ._arg(flag("examples", examples))
     }
 
     fn arg_targets_bin_example(self, bin: &'static str, example: &'static str) -> Self {
@@ -129,20 +132,23 @@ pub trait AppExt: Sized {
     }
 
     fn arg_features(self) -> Self {
-        self._arg(multi_opt(
-            "features",
-            "FEATURES",
-            "Space or comma separated list of features to activate",
-        ))
-        ._arg(opt("all-features", "Activate all available features"))
-        ._arg(opt(
+        self._arg(
+            multi_opt(
+                "features",
+                "FEATURES",
+                "Space or comma separated list of features to activate",
+            )
+            .short('F'),
+        )
+        ._arg(flag("all-features", "Activate all available features"))
+        ._arg(flag(
             "no-default-features",
             "Do not activate the `default` feature",
         ))
     }
 
     fn arg_release(self, release: &'static str) -> Self {
-        self._arg(opt("release", release))
+        self._arg(flag("release", release).short('r'))
     }
 
     fn arg_profile(self, profile: &'static str) -> Self {
@@ -150,7 +156,7 @@ pub trait AppExt: Sized {
     }
 
     fn arg_doc(self, doc: &'static str) -> Self {
-        self._arg(opt("doc", doc))
+        self._arg(flag("doc", doc))
     }
 
     fn arg_target_triple(self, target: &'static str) -> Self {
@@ -172,14 +178,14 @@ pub trait AppExt: Sized {
     }
 
     fn arg_build_plan(self) -> Self {
-        self._arg(opt(
+        self._arg(flag(
             "build-plan",
             "Output the build plan in JSON (unstable)",
         ))
     }
 
     fn arg_unit_graph(self) -> Self {
-        self._arg(opt("unit-graph", "Output build graph in JSON (unstable)"))
+        self._arg(flag("unit-graph", "Output build graph in JSON (unstable)"))
     }
 
     fn arg_new_opts(self) -> Self {
@@ -192,13 +198,13 @@ pub trait AppExt: Sized {
                  a global configuration.",
             )
             .value_name("VCS")
-            .possible_values(&["git", "hg", "pijul", "fossil", "none"]),
+            .value_parser(["git", "hg", "pijul", "fossil", "none"]),
         )
-        ._arg(opt("bin", "Use a binary (application) template [default]"))
-        ._arg(opt("lib", "Use a library template"))
+        ._arg(flag("bin", "Use a binary (application) template [default]"))
+        ._arg(flag("lib", "Use a library template"))
         ._arg(
             opt("edition", "Edition to set for the crate generated")
-                .possible_values(Edition::CLI_VALUES)
+                .value_parser(Edition::CLI_VALUES)
                 .value_name("YEAR"),
         )
         ._arg(
@@ -212,90 +218,89 @@ pub trait AppExt: Sized {
 
     fn arg_index(self) -> Self {
         self._arg(opt("index", "Registry index URL to upload the package to").value_name("INDEX"))
-            ._arg(
-                opt("host", "DEPRECATED, renamed to '--index'")
-                    .value_name("HOST")
-                    .hidden(true),
-            )
     }
 
     fn arg_dry_run(self, dry_run: &'static str) -> Self {
-        self._arg(opt("dry-run", dry_run))
+        self._arg(flag("dry-run", dry_run))
     }
 
     fn arg_ignore_rust_version(self) -> Self {
-        self._arg(opt(
+        self._arg(flag(
             "ignore-rust-version",
             "Ignore `rust-version` specification in packages",
         ))
     }
 
     fn arg_future_incompat_report(self) -> Self {
-        self._arg(opt(
+        self._arg(flag(
             "future-incompat-report",
-            "Outputs a future incompatibility report at the end of the build (unstable)",
+            "Outputs a future incompatibility report at the end of the build",
         ))
     }
+
+    fn arg_quiet(self) -> Self {
+        self._arg(flag("quiet", "Do not print cargo log messages").short('q'))
+    }
+
+    fn arg_timings(self) -> Self {
+        self._arg(
+            optional_opt(
+                "timings",
+                "Timing output formats (unstable) (comma separated): html, json",
+            )
+            .value_name("FMTS")
+            .require_equals(true),
+        )
+    }
 }
 
-impl AppExt for App {
-    fn _arg(self, arg: Arg<'static, 'static>) -> Self {
+impl CommandExt for Command {
+    fn _arg(self, arg: Arg) -> Self {
         self.arg(arg)
     }
 }
 
-pub fn opt(name: &'static str, help: &'static str) -> Arg<'static, 'static> {
-    Arg::with_name(name).long(name).help(help)
+pub fn flag(name: &'static str, help: &'static str) -> Arg {
+    Arg::new(name)
+        .long(name)
+        .help(help)
+        .action(ArgAction::SetTrue)
 }
 
-pub fn optional_opt(name: &'static str, help: &'static str) -> Arg<'static, 'static> {
-    opt(name, help).min_values(0)
+pub fn opt(name: &'static str, help: &'static str) -> Arg {
+    Arg::new(name).long(name).help(help).action(ArgAction::Set)
 }
 
-pub fn optional_multi_opt(
-    name: &'static str,
-    value_name: &'static str,
-    help: &'static str,
-) -> Arg<'static, 'static> {
+pub fn optional_opt(name: &'static str, help: &'static str) -> Arg {
+    opt(name, help).num_args(0..=1)
+}
+
+pub fn optional_multi_opt(name: &'static str, value_name: &'static str, help: &'static str) -> Arg {
     opt(name, help)
         .value_name(value_name)
-        .multiple(true)
-        .min_values(0)
-        .number_of_values(1)
+        .num_args(0..=1)
+        .action(ArgAction::Append)
 }
 
-pub fn multi_opt(
-    name: &'static str,
-    value_name: &'static str,
-    help: &'static str,
-) -> Arg<'static, 'static> {
-    // Note that all `.multiple(true)` arguments in Cargo should specify
-    // `.number_of_values(1)` as well, so that `--foo val1 val2` is
-    // *not* parsed as `foo` with values ["val1", "val2"].
-    // `number_of_values` should become the default in clap 3.
+pub fn multi_opt(name: &'static str, value_name: &'static str, help: &'static str) -> Arg {
     opt(name, help)
         .value_name(value_name)
-        .multiple(true)
-        .number_of_values(1)
+        .action(ArgAction::Append)
 }
 
-pub fn subcommand(name: &'static str) -> App {
-    SubCommand::with_name(name).settings(&[
-        AppSettings::UnifiedHelpMessage,
-        AppSettings::DeriveDisplayOrder,
-        AppSettings::DontCollapseArgsInUsage,
-    ])
+pub fn subcommand(name: &'static str) -> Command {
+    Command::new(name)
 }
 
-// Determines whether or not to gate `--profile` as unstable when resolving it.
+/// Determines whether or not to gate `--profile` as unstable when resolving it.
 pub enum ProfileChecking {
-    // `cargo rustc` historically has allowed "test", "bench", and "check". This
-    // variant explicitly allows those.
+    /// `cargo rustc` historically has allowed "test", "bench", and "check". This
+    /// variant explicitly allows those.
     LegacyRustc,
-    // `cargo check` and `cargo fix` historically has allowed "test". This variant
-    // explicitly allows that on stable.
+    /// `cargo check` and `cargo fix` historically has allowed "test". This variant
+    /// explicitly allows that on stable.
     LegacyTestOnly,
-    // All other commands, which allow any valid custom named profile.
+    /// All other commands, which allow any valid custom named profile.
     Custom,
 }
 
@@ -304,7 +309,23 @@ pub trait ArgMatchesExt {
         let arg = match self._value_of(name) {
             None => None,
             Some(arg) => Some(arg.parse::<u32>().map_err(|_| {
-                clap::Error::value_validation_auto(format!("could not parse `{}` as a number", arg))
+                clap::Error::raw(
+                    clap::error::ErrorKind::ValueValidation,
+                    format!("Invalid value: could not parse `{}` as a number", arg),
+                )
+            })?),
+        };
+        Ok(arg)
+    }
+
+    fn value_of_i32(&self, name: &str) -> CargoResult<Option<i32>> {
+        let arg = match self._value_of(name) {
+            None => None,
+            Some(arg) => Some(arg.parse::<i32>().map_err(|_| {
+                clap::Error::raw(
+                    clap::error::ErrorKind::ValueValidation,
+                    format!("Invalid value: could not parse `{}` as a number", arg),
+                )
             })?),
         };
         Ok(arg)
@@ -343,8 +364,20 @@ pub trait ArgMatchesExt {
         Ok(ws)
     }
 
-    fn jobs(&self) -> CargoResult<Option<u32>> {
-        self.value_of_u32("jobs")
+    fn jobs(&self) -> CargoResult<Option<i32>> {
+        self.value_of_i32("jobs")
+    }
+
+    fn verbose(&self) -> u32 {
+        self._count("verbose")
+    }
+
+    fn dry_run(&self) -> bool {
+        self.flag("dry-run")
+    }
+
+    fn keep_going(&self) -> bool {
+        self.flag("keep-going")
     }
 
     fn targets(&self) -> Vec<String> {
@@ -363,16 +396,22 @@ pub trait ArgMatchesExt {
         // This is an early exit, since it allows combination with `--release`.
         match (specified_profile, profile_checking) {
             // `cargo rustc` has legacy handling of these names
-            (Some(name @ ("dev" | "test" | "bench" | "check")), ProfileChecking::LegacyRustc) |
+            (Some(name @ ("dev" | "test" | "bench" | "check")), ProfileChecking::LegacyRustc)
             // `cargo fix` and `cargo check` has legacy handling of this profile name
-            (Some(name @ "test"), ProfileChecking::LegacyTestOnly) => return Ok(InternedString::new(name)),
+            | (Some(name @ "test"), ProfileChecking::LegacyTestOnly) => {
+                if self.flag("release") {
+                    config.shell().warn(
+                        "the `--release` flag should not be specified with the `--profile` flag\n\
+                         The `--release` flag will be ignored.\n\
+                         This was historically accepted, but will become an error \
+                         in a future release."
+                    )?;
+                }
+                return Ok(InternedString::new(name));
+            }
             _ => {}
         }
 
-        if specified_profile.is_some() && !config.cli_unstable().unstable_options {
-            bail!("usage of `--profile` requires `-Z unstable-options`");
-        }
-
         let conflict = |flag: &str, equiv: &str, specified: &str| -> anyhow::Error {
             anyhow::format_err!(
                 "conflicting usage of --profile={} and --{flag}\n\
@@ -384,11 +423,7 @@ pub trait ArgMatchesExt {
             )
         };
 
-        let name = match (
-            self._is_present("release"),
-            self._is_present("debug"),
-            specified_profile,
-        ) {
+        let name = match (self.flag("release"), self.flag("debug"), specified_profile) {
             (false, false, None) => default,
             (true, _, None | Some("release")) => "release",
             (true, _, Some(name)) => return Err(conflict("release", "release", name)),
@@ -414,7 +449,7 @@ pub trait ArgMatchesExt {
     fn packages_from_flags(&self) -> CargoResult<Packages> {
         Packages::from_flags(
             // TODO Integrate into 'workspace'
-            self._is_present("workspace") || self._is_present("all"),
+            self.flag("workspace") || self.flag("all"),
             self._values_of("exclude"),
             self._values_of("package"),
         )
@@ -434,25 +469,26 @@ pub trait ArgMatchesExt {
             ansi: false,
             render_diagnostics: false,
         };
+        let two_kinds_of_msg_format_err = "cannot specify two kinds of `message-format` arguments";
         for fmt in self._values_of("message-format") {
             for fmt in fmt.split(',') {
                 let fmt = fmt.to_ascii_lowercase();
                 match fmt.as_str() {
                     "json" => {
                         if message_format.is_some() {
-                            bail!("cannot specify two kinds of `message-format` arguments");
+                            bail!(two_kinds_of_msg_format_err);
                         }
                         message_format = Some(default_json);
                     }
                     "human" => {
                         if message_format.is_some() {
-                            bail!("cannot specify two kinds of `message-format` arguments");
+                            bail!(two_kinds_of_msg_format_err);
                         }
                         message_format = Some(MessageFormat::Human);
                     }
                     "short" => {
                         if message_format.is_some() {
-                            bail!("cannot specify two kinds of `message-format` arguments");
+                            bail!(two_kinds_of_msg_format_err);
                         }
                         message_format = Some(MessageFormat::Short);
                     }
@@ -464,7 +500,7 @@ pub trait ArgMatchesExt {
                             Some(MessageFormat::Json {
                                 render_diagnostics, ..
                             }) => *render_diagnostics = true,
-                            _ => bail!("cannot specify two kinds of `message-format` arguments"),
+                            _ => bail!(two_kinds_of_msg_format_err),
                         }
                     }
                     "json-diagnostic-short" => {
@@ -473,7 +509,7 @@ pub trait ArgMatchesExt {
                         }
                         match &mut message_format {
                             Some(MessageFormat::Json { short, .. }) => *short = true,
-                            _ => bail!("cannot specify two kinds of `message-format` arguments"),
+                            _ => bail!(two_kinds_of_msg_format_err),
                         }
                     }
                     "json-diagnostic-rendered-ansi" => {
@@ -482,7 +518,7 @@ pub trait ArgMatchesExt {
                         }
                         match &mut message_format {
                             Some(MessageFormat::Json { ansi, .. }) => *ansi = true,
-                            _ => bail!("cannot specify two kinds of `message-format` arguments"),
+                            _ => bail!(two_kinds_of_msg_format_err),
                         }
                     }
                     s => bail!("invalid message format specifier: `{}`", s),
@@ -490,12 +526,51 @@ pub trait ArgMatchesExt {
             }
         }
 
-        let mut build_config = BuildConfig::new(config, self.jobs()?, &self.targets(), mode)?;
+        let mut build_config = BuildConfig::new(
+            config,
+            self.jobs()?,
+            self.keep_going(),
+            &self.targets(),
+            mode,
+        )?;
         build_config.message_format = message_format.unwrap_or(MessageFormat::Human);
         build_config.requested_profile = self.get_profile_name(config, "dev", profile_checking)?;
-        build_config.build_plan = self._is_present("build-plan");
-        build_config.unit_graph = self._is_present("unit-graph");
-        build_config.future_incompat_report = self._is_present("future-incompat-report");
+        build_config.build_plan = self.flag("build-plan");
+        build_config.unit_graph = self.flag("unit-graph");
+        build_config.future_incompat_report = self.flag("future-incompat-report");
+
+        if self._contains("timings") {
+            for timing_output in self._values_of("timings") {
+                for timing_output in timing_output.split(',') {
+                    let timing_output = timing_output.to_ascii_lowercase();
+                    let timing_output = match timing_output.as_str() {
+                        "html" => {
+                            config
+                                .cli_unstable()
+                                .fail_if_stable_opt("--timings=html", 7405)?;
+                            TimingOutput::Html
+                        }
+                        "json" => {
+                            config
+                                .cli_unstable()
+                                .fail_if_stable_opt("--timings=json", 7405)?;
+                            TimingOutput::Json
+                        }
+                        s => bail!("invalid timings output specifier: `{}`", s),
+                    };
+                    build_config.timing_outputs.push(timing_output);
+                }
+            }
+            if build_config.timing_outputs.is_empty() {
+                build_config.timing_outputs.push(TimingOutput::Html);
+            }
+        }
+
+        if build_config.keep_going {
+            config
+                .cli_unstable()
+                .fail_if_stable_opt("--keep-going", 10496)?;
+        }
         if build_config.build_plan {
             config
                 .cli_unstable()
@@ -506,39 +581,28 @@ pub trait ArgMatchesExt {
                 .cli_unstable()
                 .fail_if_stable_opt("--unit-graph", 8002)?;
         }
-        if build_config.future_incompat_report {
-            config
-                .cli_unstable()
-                .fail_if_stable_opt("--future-incompat-report", 9241)?;
-
-            if !config.cli_unstable().future_incompat_report {
-                anyhow::bail!(
-                    "Usage of `--future-incompat-report` requires `-Z future-incompat-report`"
-                )
-            }
-        }
 
         let opts = CompileOptions {
             build_config,
             cli_features: self.cli_features()?,
             spec,
             filter: CompileFilter::from_raw_arguments(
-                self._is_present("lib"),
+                self.flag("lib"),
                 self._values_of("bin"),
-                self._is_present("bins"),
+                self.flag("bins"),
                 self._values_of("test"),
-                self._is_present("tests"),
+                self.flag("tests"),
                 self._values_of("example"),
-                self._is_present("examples"),
+                self.flag("examples"),
                 self._values_of("bench"),
-                self._is_present("benches"),
-                self._is_present("all-targets"),
+                self.flag("benches"),
+                self.flag("all-targets"),
             ),
             target_rustdoc_args: None,
             target_rustc_args: None,
-            local_rustdoc_args: None,
+            target_rustc_crate_types: None,
             rustdoc_document_private_items: false,
-            honor_rust_version: !self._is_present("ignore-rust-version"),
+            honor_rust_version: !self.flag("ignore-rust-version"),
         };
 
         if let Some(ws) = workspace {
@@ -559,8 +623,8 @@ pub trait ArgMatchesExt {
     fn cli_features(&self) -> CargoResult<CliFeatures> {
         CliFeatures::from_command_line(
             &self._values_of("features"),
-            self._is_present("all-features"),
-            !self._is_present("no-default-features"),
+            self.flag("all-features"),
+            !self.flag("no-default-features"),
         )
     }
 
@@ -591,8 +655,8 @@ pub trait ArgMatchesExt {
         });
         NewOptions::new(
             vcs,
-            self._is_present("bin"),
-            self._is_present("lib"),
+            self.flag("bin"),
+            self.flag("lib"),
             self.value_of_path("path", config).unwrap(),
             self._value_of("name").map(|s| s.to_string()),
             self._value_of("edition").map(|s| s.to_string()),
@@ -601,46 +665,27 @@ pub trait ArgMatchesExt {
     }
 
     fn registry(&self, config: &Config) -> CargoResult<Option<String>> {
-        match self._value_of("registry") {
-            Some(registry) => {
-                validate_package_name(registry, "registry name", "")?;
-
-                if registry == CRATES_IO_REGISTRY {
-                    // If "crates.io" is specified, then we just need to return `None`,
-                    // as that will cause cargo to use crates.io. This is required
-                    // for the case where a default alternative registry is used
-                    // but the user wants to switch back to crates.io for a single
-                    // command.
-                    Ok(None)
-                } else {
-                    Ok(Some(registry.to_string()))
-                }
+        let registry = self._value_of("registry");
+        let index = self._value_of("index");
+        let result = match (registry, index) {
+            (None, None) => config.default_registry()?,
+            (None, Some(_)) => {
+                // If --index is set, then do not look at registry.default.
+                None
             }
-            None => config.default_registry(),
-        }
+            (Some(r), None) => {
+                validate_package_name(r, "registry name", "")?;
+                Some(r.to_string())
+            }
+            (Some(_), Some(_)) => {
+                bail!("both `--index` and `--registry` should not be set at the same time")
+            }
+        };
+        Ok(result)
     }
 
-    fn index(&self, config: &Config) -> CargoResult<Option<String>> {
-        // TODO: deprecated. Remove once it has been decided `--host` can be removed
-        // We may instead want to repurpose the host flag, as mentioned in issue
-        // rust-lang/cargo#4208.
-        let msg = "The flag '--host' is no longer valid.
-
-Previous versions of Cargo accepted this flag, but it is being
-deprecated. The flag is being renamed to 'index', as the flag
-wants the location of the index. Please use '--index' instead.
-
-This will soon become a hard error, so it's either recommended
-to update to a fixed version or contact the upstream maintainer
-about this warning.";
-
-        let index = match self._value_of("host") {
-            Some(host) => {
-                config.shell().warn(&msg)?;
-                Some(host.to_string())
-            }
-            None => self._value_of("index").map(|s| s.to_string()),
-        };
+    fn index(&self) -> CargoResult<Option<String>> {
+        let index = self._value_of("index").map(|s| s.to_string());
         Ok(index)
     }
 
@@ -673,9 +718,11 @@ about this warning.";
     }
 
     fn is_present_with_zero_values(&self, name: &str) -> bool {
-        self._is_present(name) && self._value_of(name).is_none()
+        self._contains(name) && self._value_of(name).is_none()
     }
 
+    fn flag(&self, name: &str) -> bool;
+
     fn _value_of(&self, name: &str) -> Option<&str>;
 
     fn _values_of(&self, name: &str) -> Vec<String>;
@@ -684,45 +731,68 @@ about this warning.";
 
     fn _values_of_os(&self, name: &str) -> Vec<OsString>;
 
-    fn _is_present(&self, name: &str) -> bool;
+    fn _count(&self, name: &str) -> u32;
+
+    fn _contains(&self, name: &str) -> bool;
 }
 
-impl<'a> ArgMatchesExt for ArgMatches<'a> {
+impl<'a> ArgMatchesExt for ArgMatches {
+    fn flag(&self, name: &str) -> bool {
+        ignore_unknown(self.try_get_one::<bool>(name))
+            .copied()
+            .unwrap_or(false)
+    }
+
     fn _value_of(&self, name: &str) -> Option<&str> {
-        self.value_of(name)
+        ignore_unknown(self.try_get_one::<String>(name)).map(String::as_str)
     }
 
     fn _value_of_os(&self, name: &str) -> Option<&OsStr> {
-        self.value_of_os(name)
+        ignore_unknown(self.try_get_one::<OsString>(name)).map(OsString::as_os_str)
     }
 
     fn _values_of(&self, name: &str) -> Vec<String> {
-        self.values_of(name)
+        ignore_unknown(self.try_get_many::<String>(name))
             .unwrap_or_default()
-            .map(|s| s.to_string())
+            .cloned()
             .collect()
     }
 
     fn _values_of_os(&self, name: &str) -> Vec<OsString> {
-        self.values_of_os(name)
+        ignore_unknown(self.try_get_many::<OsString>(name))
             .unwrap_or_default()
-            .map(|s| s.to_os_string())
+            .cloned()
             .collect()
     }
 
-    fn _is_present(&self, name: &str) -> bool {
-        self.is_present(name)
+    fn _count(&self, name: &str) -> u32 {
+        *ignore_unknown(self.try_get_one::<u8>(name)).expect("defaulted by clap") as u32
+    }
+
+    fn _contains(&self, name: &str) -> bool {
+        ignore_unknown(self.try_contains_id(name))
     }
 }
 
-pub fn values(args: &ArgMatches<'_>, name: &str) -> Vec<String> {
+pub fn values(args: &ArgMatches, name: &str) -> Vec<String> {
     args._values_of(name)
 }
 
-pub fn values_os(args: &ArgMatches<'_>, name: &str) -> Vec<OsString> {
+pub fn values_os(args: &ArgMatches, name: &str) -> Vec<OsString> {
     args._values_of_os(name)
 }
 
+#[track_caller]
+pub fn ignore_unknown<T: Default>(r: Result<T, clap::parser::MatchesError>) -> T {
+    match r {
+        Ok(t) => t,
+        Err(clap::parser::MatchesError::UnknownArgument { .. }) => Default::default(),
+        Err(e) => {
+            panic!("Mismatch between definition and access: {}", e);
+        }
+    }
+}
+
 #[derive(PartialEq, Eq, PartialOrd, Ord)]
 pub enum CommandInfo {
     BuiltIn { about: Option<String> },
