@@ -119,7 +119,7 @@ fn load_config_table(config: &Config, prefix: &str) -> CargoResult<TargetConfig>
     // Links do not support environment variables.
     let target_key = ConfigKey::from_str(prefix);
     let links_overrides = match config.get_table(&target_key)? {
-        Some(links) => parse_links_overrides(&target_key, links.val)?,
+        Some(links) => parse_links_overrides(&target_key, links.val, config)?,
         None => BTreeMap::new(),
     };
     Ok(TargetConfig {
@@ -133,8 +133,14 @@ fn load_config_table(config: &Config, prefix: &str) -> CargoResult<TargetConfig>
 fn parse_links_overrides(
     target_key: &ConfigKey,
     links: HashMap<String, CV>,
+    config: &Config,
 ) -> CargoResult<BTreeMap<String, BuildOutput>> {
     let mut links_overrides = BTreeMap::new();
+    let extra_check_cfg = match config.cli_unstable().check_cfg {
+        Some((_, _, _, output)) => output,
+        None => false,
+    };
+
     for (lib_name, value) in links {
         // Skip these keys, it shares the namespace with `TargetConfig`.
         match lib_name.as_str() {
@@ -172,24 +178,44 @@ fn parse_links_overrides(
                         .extend(list.iter().map(|v| PathBuf::from(&v.0)));
                 }
                 "rustc-link-arg-cdylib" | "rustc-cdylib-link-arg" => {
-                    let args = value.list(key)?;
-                    let args = args.iter().map(|v| (LinkType::Cdylib, v.0.clone()));
+                    let args = extra_link_args(LinkType::Cdylib, key, value)?;
                     output.linker_args.extend(args);
                 }
                 "rustc-link-arg-bins" => {
-                    let args = value.list(key)?;
-                    let args = args.iter().map(|v| (LinkType::Bin, v.0.clone()));
+                    let args = extra_link_args(LinkType::Bin, key, value)?;
                     output.linker_args.extend(args);
                 }
                 "rustc-link-arg" => {
-                    let args = value.list(key)?;
-                    let args = args.iter().map(|v| (LinkType::All, v.0.clone()));
+                    let args = extra_link_args(LinkType::All, key, value)?;
+                    output.linker_args.extend(args);
+                }
+                "rustc-link-arg-tests" => {
+                    let args = extra_link_args(LinkType::Test, key, value)?;
+                    output.linker_args.extend(args);
+                }
+                "rustc-link-arg-benches" => {
+                    let args = extra_link_args(LinkType::Bench, key, value)?;
+                    output.linker_args.extend(args);
+                }
+                "rustc-link-arg-examples" => {
+                    let args = extra_link_args(LinkType::Example, key, value)?;
                     output.linker_args.extend(args);
                 }
                 "rustc-cfg" => {
                     let list = value.list(key)?;
                     output.cfgs.extend(list.iter().map(|v| v.0.clone()));
                 }
+                "rustc-check-cfg" => {
+                    if extra_check_cfg {
+                        let list = value.list(key)?;
+                        output.check_cfgs.extend(list.iter().map(|v| v.0.clone()));
+                    } else {
+                        config.shell().warn(format!(
+                            "target config `{}.{}` requires -Zcheck-cfg=output flag",
+                            target_key, key
+                        ))?;
+                    }
+                }
                 "rustc-env" => {
                     for (name, val) in value.table(key)?.0 {
                         let val = val.string(name)?.0;
@@ -209,3 +235,12 @@ fn parse_links_overrides(
     }
     Ok(links_overrides)
 }
+
+fn extra_link_args<'a>(
+    link_type: LinkType,
+    key: &str,
+    value: &'a CV,
+) -> CargoResult<impl Iterator<Item = (LinkType, String)> + 'a> {
+    let args = value.list(key)?;
+    Ok(args.iter().map(move |v| (link_type.clone(), v.0.clone())))
+}
