@@ -1,7 +1,7 @@
 //! Tests for `cargo-features` definitions.
 
 use cargo_test_support::registry::Package;
-use cargo_test_support::{is_nightly, project, registry};
+use cargo_test_support::{project, registry};
 
 #[cargo_test]
 fn feature_required() {
@@ -18,8 +18,8 @@ fn feature_required() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("check")
+        .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_status(101)
         .with_stderr(
             "\
@@ -42,7 +42,7 @@ Caused by:
         .run();
 
     // Same, but stable.
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -97,8 +97,8 @@ fn feature_required_dependency() {
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("check")
+        .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_status(101)
         .with_stderr(
             "\
@@ -126,7 +126,7 @@ Caused by:
         .run();
 
     // Same, but stable.
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -173,7 +173,7 @@ fn unknown_feature() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -202,27 +202,21 @@ fn stable_feature_warns() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build")
+    p.cargo("check")
         .with_stderr(
             "\
 warning: the cargo feature `test-dummy-stable` has been stabilized in the 1.0 \
 release and is no longer necessary to be listed in the manifest
   See https://doc.rust-lang.org/[..]cargo/ for more information about using this feature.
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "-Zallow-features is unstable")]
 fn allow_features() {
-    if !is_nightly() {
-        // -Zallow-features on rustc is nightly only
-        eprintln!("skipping test allow_features without nightly rustc");
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
@@ -239,23 +233,31 @@ fn allow_features() {
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("-Zallow-features=test-dummy-unstable build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("-Zallow-features=test-dummy-unstable check")
+        .masquerade_as_nightly_cargo(&["allow-features", "test-dummy-unstable"])
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
         .run();
 
-    p.cargo("-Zallow-features=test-dummy-unstable,print-im-a-teapot -Zprint-im-a-teapot build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("-Zallow-features=test-dummy-unstable,print-im-a-teapot -Zprint-im-a-teapot check")
+        .masquerade_as_nightly_cargo(&[
+            "allow-features",
+            "test-dummy-unstable",
+            "print-im-a-teapot",
+        ])
         .with_stdout("im-a-teapot = true")
         .run();
 
-    p.cargo("-Zallow-features=test-dummy-unstable -Zprint-im-a-teapot build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("-Zallow-features=test-dummy-unstable -Zprint-im-a-teapot check")
+        .masquerade_as_nightly_cargo(&[
+            "allow-features",
+            "test-dummy-unstable",
+            "print-im-a-teapot",
+        ])
         .with_status(101)
         .with_stderr(
             "\
@@ -264,8 +266,8 @@ error: the feature `print-im-a-teapot` is not in the list of allowed features: [
         )
         .run();
 
-    p.cargo("-Zallow-features= build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("-Zallow-features= check")
+        .masquerade_as_nightly_cargo(&["allow-features", "test-dummy-unstable"])
         .with_status(101)
         .with_stderr(
             "\
@@ -278,14 +280,8 @@ Caused by:
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "-Zallow-features is unstable")]
 fn allow_features_to_rustc() {
-    if !is_nightly() {
-        // -Zallow-features on rustc is nightly only
-        eprintln!("skipping test allow_features_to_rustc without nightly rustc");
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
@@ -304,31 +300,25 @@ fn allow_features_to_rustc() {
         )
         .build();
 
-    p.cargo("-Zallow-features= build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("-Zallow-features= check")
+        .masquerade_as_nightly_cargo(&["allow-features"])
         .with_status(101)
         .with_stderr_contains("[..]E0725[..]")
         .run();
 
-    p.cargo("-Zallow-features=test_2018_feature build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("-Zallow-features=test_2018_feature check")
+        .masquerade_as_nightly_cargo(&["allow-features"])
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "-Zallow-features is unstable")]
 fn allow_features_in_cfg() {
-    if !is_nightly() {
-        // -Zallow-features on rustc is nightly only
-        eprintln!("skipping test allow_features_in_cfg without nightly rustc");
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
@@ -352,24 +342,32 @@ fn allow_features_in_cfg() {
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("check")
+        .masquerade_as_nightly_cargo(&[
+            "allow-features",
+            "test-dummy-unstable",
+            "print-im-a-teapot",
+        ])
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
         .run();
 
-    p.cargo("-Zprint-im-a-teapot build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("-Zprint-im-a-teapot check")
+        .masquerade_as_nightly_cargo(&[
+            "allow-features",
+            "test-dummy-unstable",
+            "print-im-a-teapot",
+        ])
         .with_stdout("im-a-teapot = true")
         .with_stderr("[FINISHED] [..]")
         .run();
 
-    p.cargo("-Zunstable-options build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("-Zunstable-options check")
+        .masquerade_as_nightly_cargo(&["allow-features", "test-dummy-unstable", "print-im-a-teapot"])
         .with_status(101)
         .with_stderr(
             "\
@@ -379,8 +377,12 @@ error: the feature `unstable-options` is not in the list of allowed features: [p
         .run();
 
     // -Zallow-features overrides .cargo/config
-    p.cargo("-Zallow-features=test-dummy-unstable -Zprint-im-a-teapot build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("-Zallow-features=test-dummy-unstable -Zprint-im-a-teapot check")
+        .masquerade_as_nightly_cargo(&[
+            "allow-features",
+            "test-dummy-unstable",
+            "print-im-a-teapot",
+        ])
         .with_status(101)
         .with_stderr(
             "\
@@ -389,8 +391,12 @@ error: the feature `print-im-a-teapot` is not in the list of allowed features: [
         )
         .run();
 
-    p.cargo("-Zallow-features= build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("-Zallow-features= check")
+        .masquerade_as_nightly_cargo(&[
+            "allow-features",
+            "test-dummy-unstable",
+            "print-im-a-teapot",
+        ])
         .with_status(101)
         .with_stderr(
             "\
@@ -420,17 +426,17 @@ fn nightly_feature_requires_nightly() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("check")
+        .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
         .run();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -477,18 +483,18 @@ fn nightly_feature_requires_nightly_in_dep() {
         )
         .file("a/src/lib.rs", "")
         .build();
-    p.cargo("build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("check")
+        .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_stderr(
             "\
-[COMPILING] a [..]
-[COMPILING] b [..]
+[CHECKING] a [..]
+[CHECKING] b [..]
 [FINISHED] [..]
 ",
         )
         .run();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -531,17 +537,17 @@ fn cant_publish() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build")
-        .masquerade_as_nightly_cargo()
+    p.cargo("check")
+        .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
         .run();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -575,7 +581,7 @@ fn z_flags_rejected() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build -Zprint-im-a-teapot")
+    p.cargo("check -Zprint-im-a-teapot")
         .with_status(101)
         .with_stderr(
             "error: the `-Z` flag is only accepted on the nightly \
@@ -584,18 +590,18 @@ fn z_flags_rejected() {
         )
         .run();
 
-    p.cargo("build -Zarg")
-        .masquerade_as_nightly_cargo()
+    p.cargo("check -Zarg")
+        .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_status(101)
         .with_stderr("error: unknown `-Z` flag specified: arg")
         .run();
 
-    p.cargo("build -Zprint-im-a-teapot")
-        .masquerade_as_nightly_cargo()
+    p.cargo("check -Zprint-im-a-teapot")
+        .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_stdout("im-a-teapot = true\n")
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
@@ -604,7 +610,10 @@ fn z_flags_rejected() {
 
 #[cargo_test]
 fn publish_allowed() {
-    registry::init();
+    let registry = registry::RegistryBuilder::new()
+        .http_api()
+        .http_index()
+        .build();
 
     let p = project()
         .file(
@@ -620,8 +629,24 @@ fn publish_allowed() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("publish --token sekrit")
-        .masquerade_as_nightly_cargo()
+
+    p.cargo("publish")
+        .replace_crates_io(registry.index_url())
+        .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
+        .with_stderr(
+            "\
+[UPDATING] [..]
+[WARNING] [..]
+[..]
+[PACKAGING] a v0.0.1 [..]
+[VERIFYING] a v0.0.1 [..]
+[COMPILING] a v0.0.1 [..]
+[FINISHED] [..]
+[PACKAGED] [..]
+[UPLOADING] a v0.0.1 [..]
+[UPDATING] [..]
+",
+        )
         .run();
 }
 
@@ -640,7 +665,7 @@ fn wrong_position() {
         .file("src/lib.rs", "")
         .build();
     p.cargo("check")
-        .masquerade_as_nightly_cargo()
+        .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_status(101)
         .with_stderr(
             "\
@@ -659,7 +684,7 @@ fn z_stabilized() {
     let p = project().file("src/lib.rs", "").build();
 
     p.cargo("check -Z cache-messages")
-        .masquerade_as_nightly_cargo()
+        .masquerade_as_nightly_cargo(&["always_nightly"])
         .with_stderr(
             "\
 warning: flag `-Z cache-messages` has been stabilized in the 1.40 release, \
@@ -673,7 +698,7 @@ warning: flag `-Z cache-messages` has been stabilized in the 1.40 release, \
         .run();
 
     p.cargo("check -Z offline")
-        .masquerade_as_nightly_cargo()
+        .masquerade_as_nightly_cargo(&["always_nightly"])
         .with_status(101)
         .with_stderr(
             "\
