@@ -19,17 +19,17 @@ fn override_simple() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            bar = "0.1.0"
+                    [dependencies]
+                    bar = "0.1.0"
 
-            [replace]
-            "bar:0.1.0" = {{ git = '{}' }}
-        "#,
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{}' }}
+                "#,
                 bar.url()
             ),
         )
@@ -39,13 +39,111 @@ fn override_simple() {
         )
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_stderr(
             "\
-[UPDATING] `[ROOT][..]` index
+[UPDATING] `dummy-registry` index
 [UPDATING] git repository `[..]`
-[COMPILING] bar v0.1.0 (file://[..])
-[COMPILING] foo v0.0.1 ([CWD])
+[CHECKING] bar v0.1.0 (file://[..])
+[CHECKING] foo v0.0.1 ([CWD])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn override_with_features() {
+    Package::new("bar", "0.1.0").publish();
+
+    let bar = git::repo(&paths::root().join("override"))
+        .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("src/lib.rs", "pub fn bar() {}")
+        .build();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
+
+                    [dependencies]
+                    bar = "0.1.0"
+
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{}', features = ["some_feature"] }}
+                "#,
+                bar.url()
+            ),
+        )
+        .file(
+            "src/lib.rs",
+            "extern crate bar; pub fn foo() { bar::bar(); }",
+        )
+        .build();
+
+    p.cargo("check")
+        .with_stderr(
+            "\
+[UPDATING] [..] index
+[UPDATING] git repository `[..]`
+[WARNING] replacement for `bar` uses the features mechanism. default-features and features \
+will not take effect because the replacement dependency does not support this mechanism
+[CHECKING] bar v0.1.0 (file://[..])
+[CHECKING] foo v0.0.1 ([CWD])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn override_with_setting_default_features() {
+    Package::new("bar", "0.1.0").publish();
+
+    let bar = git::repo(&paths::root().join("override"))
+        .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("src/lib.rs", "pub fn bar() {}")
+        .build();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
+
+                    [dependencies]
+                    bar = "0.1.0"
+
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{}', default-features = false, features = ["none_default_feature"] }}
+                "#,
+                bar.url()
+            ),
+        )
+        .file(
+            "src/lib.rs",
+            "extern crate bar; pub fn foo() { bar::bar(); }",
+        )
+        .build();
+
+    p.cargo("check")
+        .with_stderr(
+            "\
+[UPDATING] [..] index
+[UPDATING] git repository `[..]`
+[WARNING] replacement for `bar` uses the features mechanism. default-features and features \
+will not take effect because the replacement dependency does not support this mechanism
+[CHECKING] bar v0.1.0 (file://[..])
+[CHECKING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
         )
@@ -58,22 +156,22 @@ fn missing_version() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies]
-            bar = "0.1.0"
+                [dependencies]
+                bar = "0.1.0"
 
-            [replace]
-            bar = { git = 'https://example.com' }
-        "#,
+                [replace]
+                bar = { git = 'https://example.com' }
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -92,22 +190,22 @@ fn invalid_semver_version() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies]
-            bar = "*"
+                [dependencies]
+                bar = "*"
 
-            [replace]
-            "bar:*" = { git = 'https://example.com' }
-        "#,
+                [replace]
+                "bar:*" = { git = 'https://example.com' }
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr_contains(
             "\
@@ -129,22 +227,22 @@ fn different_version() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies]
-            bar = "0.1.0"
+                [dependencies]
+                bar = "0.1.0"
 
-            [replace]
-            "bar:0.1.0" = "0.2.0"
-        "#,
+                [replace]
+                "bar:0.1.0" = "0.2.0"
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -175,39 +273,39 @@ fn transitive() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            baz = "0.2.0"
+                    [dependencies]
+                    baz = "0.2.0"
 
-            [replace]
-            "bar:0.1.0" = {{ git = '{}' }}
-        "#,
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{}' }}
+                "#,
                 foo.url()
             ),
         )
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_stderr(
             "\
-[UPDATING] `[ROOT][..]` index
+[UPDATING] `dummy-registry` index
 [UPDATING] git repository `[..]`
 [DOWNLOADING] crates ...
 [DOWNLOADED] baz v0.2.0 (registry [..])
-[COMPILING] bar v0.1.0 (file://[..])
-[COMPILING] baz v0.2.0
-[COMPILING] foo v0.0.1 ([CWD])
+[CHECKING] bar v0.1.0 (file://[..])
+[CHECKING] baz v0.2.0
+[CHECKING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
         )
         .run();
 
-    p.cargo("build").with_stdout("").run();
+    p.cargo("check").with_stdout("").run();
 }
 
 #[cargo_test]
@@ -224,17 +322,17 @@ fn persists_across_rebuilds() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            bar = "0.1.0"
+                    [dependencies]
+                    bar = "0.1.0"
 
-            [replace]
-            "bar:0.1.0" = {{ git = '{}' }}
-        "#,
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{}' }}
+                "#,
                 foo.url()
             ),
         )
@@ -244,19 +342,19 @@ fn persists_across_rebuilds() {
         )
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_stderr(
             "\
-[UPDATING] `[ROOT][..]` index
+[UPDATING] `dummy-registry` index
 [UPDATING] git repository `file://[..]`
-[COMPILING] bar v0.1.0 (file://[..])
-[COMPILING] foo v0.0.1 ([CWD])
+[CHECKING] bar v0.1.0 (file://[..])
+[CHECKING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
         )
         .run();
 
-    p.cargo("build").with_stdout("").run();
+    p.cargo("check").with_stdout("").run();
 }
 
 #[cargo_test]
@@ -273,17 +371,17 @@ fn replace_registry_with_path() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies]
-            bar = "0.1.0"
+                [dependencies]
+                bar = "0.1.0"
 
-            [replace]
-            "bar:0.1.0" = { path = "../bar" }
-        "#,
+                [replace]
+                "bar:0.1.0" = { path = "../bar" }
+            "#,
         )
         .file(
             "src/lib.rs",
@@ -291,12 +389,12 @@ fn replace_registry_with_path() {
         )
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_stderr(
             "\
-[UPDATING] `[ROOT][..]` index
-[COMPILING] bar v0.1.0 ([ROOT][..])
-[COMPILING] foo v0.0.1 ([CWD])
+[UPDATING] `dummy-registry` index
+[CHECKING] bar v0.1.0 ([ROOT][..]/bar)
+[CHECKING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
         )
@@ -327,18 +425,18 @@ fn use_a_spec_to_select() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            bar = "0.1"
-            baz = "0.1"
+                    [dependencies]
+                    bar = "0.1"
+                    baz = "0.1"
 
-            [replace]
-            "baz:0.2.0" = {{ git = '{}' }}
-        "#,
+                    [replace]
+                    "baz:0.2.0" = {{ git = '{}' }}
+                "#,
                 foo.url()
             ),
         )
@@ -356,18 +454,18 @@ fn use_a_spec_to_select() {
         )
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_stderr(
             "\
-[UPDATING] `[ROOT][..]` index
+[UPDATING] `dummy-registry` index
 [UPDATING] git repository `[..]`
 [DOWNLOADING] crates ...
 [DOWNLOADED] [..]
 [DOWNLOADED] [..]
-[COMPILING] [..]
-[COMPILING] [..]
-[COMPILING] [..]
-[COMPILING] foo v0.0.1 ([CWD])
+[CHECKING] [..]
+[CHECKING] [..]
+[CHECKING] [..]
+[CHECKING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
         )
@@ -383,14 +481,14 @@ fn override_adds_some_deps() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
+                [package]
+                name = "bar"
+                version = "0.1.0"
+                authors = []
 
-            [dependencies]
-            baz = "0.1"
-        "#,
+                [dependencies]
+                baz = "0.1"
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -400,39 +498,39 @@ fn override_adds_some_deps() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            bar = "0.1"
+                    [dependencies]
+                    bar = "0.1"
 
-            [replace]
-            "bar:0.1.0" = {{ git = '{}' }}
-        "#,
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{}' }}
+                "#,
                 foo.url()
             ),
         )
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_stderr(
             "\
-[UPDATING] `[ROOT][..]` index
+[UPDATING] `dummy-registry` index
 [UPDATING] git repository `[..]`
 [DOWNLOADING] crates ...
 [DOWNLOADED] baz v0.1.1 (registry [..])
-[COMPILING] baz v0.1.1
-[COMPILING] bar v0.1.0 ([..])
-[COMPILING] foo v0.0.1 ([CWD])
+[CHECKING] baz v0.1.1
+[CHECKING] bar v0.1.0 ([..])
+[CHECKING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
         )
         .run();
 
-    p.cargo("build").with_stdout("").run();
+    p.cargo("check").with_stdout("").run();
 
     Package::new("baz", "0.1.2").publish();
     p.cargo("update -p")
@@ -440,19 +538,19 @@ fn override_adds_some_deps() {
         .with_stderr(
             "\
 [UPDATING] git repository `file://[..]`
-[UPDATING] `[ROOT][..]` index
+[UPDATING] `dummy-registry` index
 ",
         )
         .run();
     p.cargo("update -p https://github.com/rust-lang/crates.io-index#bar")
         .with_stderr(
             "\
-[UPDATING] `[ROOT][..]` index
+[UPDATING] `dummy-registry` index
 ",
         )
         .run();
 
-    p.cargo("build").with_stdout("").run();
+    p.cargo("check").with_stdout("").run();
 }
 
 #[cargo_test]
@@ -466,14 +564,14 @@ fn locked_means_locked_yes_no_seriously_i_mean_locked() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
+                [package]
+                name = "bar"
+                version = "0.1.0"
+                authors = []
 
-            [dependencies]
-            baz = "*"
-        "#,
+                [dependencies]
+                baz = "*"
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -483,28 +581,28 @@ fn locked_means_locked_yes_no_seriously_i_mean_locked() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            bar = "0.1"
-            baz = "0.1"
+                    [dependencies]
+                    bar = "0.1"
+                    baz = "0.1"
 
-            [replace]
-            "bar:0.1.0" = {{ git = '{}' }}
-        "#,
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{}' }}
+                "#,
                 foo.url()
             ),
         )
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build").run();
+    p.cargo("check").run();
 
-    p.cargo("build").with_stdout("").run();
-    p.cargo("build").with_stdout("").run();
+    p.cargo("check").with_stdout("").run();
+    p.cargo("check").with_stdout("").run();
 }
 
 #[cargo_test]
@@ -521,32 +619,35 @@ fn override_wrong_name() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            baz = "0.1"
+                    [dependencies]
+                    baz = "0.1"
 
-            [replace]
-            "baz:0.1.0" = {{ git = '{}' }}
-        "#,
+                    [replace]
+                    "baz:0.1.0" = {{ git = '{}' }}
+                "#,
                 foo.url()
             ),
         )
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
 [UPDATING] [..] index
 [UPDATING] git repository [..]
-error: no matching package for override `[..]baz:0.1.0` found
-location searched: file://[..]
-version required: = 0.1.0
+[ERROR] failed to get `baz` as a dependency of package `foo v0.0.1 ([..])`
+
+Caused by:
+  no matching package for override `[..]baz@0.1.0` found
+  location searched: file://[..]
+  version required: =0.1.0
 ",
         )
         .run();
@@ -565,30 +666,33 @@ fn override_with_nothing() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            bar = "0.1"
+                    [dependencies]
+                    bar = "0.1"
 
-            [replace]
-            "bar:0.1.0" = {{ git = '{}' }}
-        "#,
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{}' }}
+                "#,
                 foo.url()
             ),
         )
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
 [UPDATING] [..] index
 [UPDATING] git repository [..]
-[ERROR] failed to load source for a dependency on `bar`
+[ERROR] failed to get `bar` as a dependency of package `foo v0.0.1 ([..])`
+
+Caused by:
+  failed to load source for dependency `bar`
 
 Caused by:
   Unable to update file://[..]
@@ -606,26 +710,26 @@ fn override_wrong_version() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [replace]
-            "bar:0.1.0" = { git = 'https://example.com', version = '0.2.0' }
-        "#,
+                [replace]
+                "bar:0.1.0" = { git = 'https://example.com', version = '0.2.0' }
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
 error: failed to parse manifest at `[..]`
 
 Caused by:
-  replacements cannot specify a version requirement, but found one for `[..]bar:0.1.0`
+  replacements cannot specify a version requirement, but found one for `[..]bar@0.1.0`
 ",
         )
         .run();
@@ -645,38 +749,41 @@ fn multiple_specs() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            bar = "0.1.0"
+                    [dependencies]
+                    bar = "0.1.0"
 
-            [replace]
-            "bar:0.1.0" = {{ git = '{0}' }}
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{0}' }}
 
-            [replace."https://github.com/rust-lang/crates.io-index#bar:0.1.0"]
-            git = '{0}'
-        "#,
+                    [replace."https://github.com/rust-lang/crates.io-index#bar:0.1.0"]
+                    git = '{0}'
+                "#,
                 bar.url()
             ),
         )
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
 [UPDATING] [..] index
 [UPDATING] git repository [..]
-error: overlapping replacement specifications found:
+[ERROR] failed to get `bar` as a dependency of package `foo v0.0.1 ([..])`
 
-  * [..]
-  * [..]
+Caused by:
+  overlapping replacement specifications found:
 
-both specifications match: bar v0.1.0
+    * [..]
+    * [..]
+
+  both specifications match: bar v0.1.0
 ",
         )
         .run();
@@ -696,17 +803,17 @@ fn test_override_dep() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            bar = "0.1.0"
+                    [dependencies]
+                    bar = "0.1.0"
 
-            [replace]
-            "bar:0.1.0" = {{ git = '{0}' }}
-        "#,
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{0}' }}
+                "#,
                 bar.url()
             ),
         )
@@ -719,8 +826,8 @@ fn test_override_dep() {
             "\
 error: There are multiple `bar` packages in your project, and the [..]
 Please re-run this command with [..]
-  [..]#bar:0.1.0
-  [..]#bar:0.1.0
+  [..]#bar@0.1.0
+  [..]#bar@0.1.0
 ",
         )
         .run();
@@ -740,17 +847,17 @@ fn update() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            bar = "0.1.0"
+                    [dependencies]
+                    bar = "0.1.0"
 
-            [replace]
-            "bar:0.1.0" = {{ git = '{0}' }}
-        "#,
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{0}' }}
+                "#,
                 bar.url()
             ),
         )
@@ -778,14 +885,14 @@ fn no_override_self() {
         .file(
             "near/Cargo.toml",
             r#"
-            [package]
-            name = "near"
-            version = "0.1.0"
-            authors = []
+                [package]
+                name = "near"
+                version = "0.1.0"
+                authors = []
 
-            [dependencies]
-            far = { path = "../far" }
-        "#,
+                [dependencies]
+                far = { path = "../far" }
+            "#,
         )
         .file("near/src/lib.rs", "#![no_std] pub extern crate far;")
         .build();
@@ -795,24 +902,24 @@ fn no_override_self() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            near = {{ git = '{0}' }}
+                    [dependencies]
+                    near = {{ git = '{0}' }}
 
-            [replace]
-            "near:0.1.0" = {{ git = '{0}' }}
-        "#,
+                    [replace]
+                    "near:0.1.0" = {{ git = '{0}' }}
+                "#,
                 deps.url()
             ),
         )
         .file("src/lib.rs", "#![no_std] pub extern crate near;")
         .build();
 
-    p.cargo("build --verbose").run();
+    p.cargo("check --verbose").run();
 }
 
 #[cargo_test]
@@ -831,59 +938,59 @@ fn override_an_override() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies]
-            chrono = "0.2"
-            serde = "0.8"
+                [dependencies]
+                chrono = "0.2"
+                serde = "0.8"
 
-            [replace]
-            "chrono:0.2.0" = { path = "chrono" }
-            "serde:0.8.0" = { path = "serde" }
-        "#,
+                [replace]
+                "chrono:0.2.0" = { path = "chrono" }
+                "serde:0.8.0" = { path = "serde" }
+            "#,
         )
         .file(
             "Cargo.lock",
             r#"
-            [[package]]
-            name = "foo"
-            version = "0.0.1"
-            dependencies = [
-             "chrono 0.2.0 (registry+https://github.com/rust-lang/crates.io-index)",
-             "serde 0.8.0 (registry+https://github.com/rust-lang/crates.io-index)",
-            ]
+                [[package]]
+                name = "foo"
+                version = "0.0.1"
+                dependencies = [
+                 "chrono 0.2.0 (registry+https://github.com/rust-lang/crates.io-index)",
+                 "serde 0.8.0 (registry+https://github.com/rust-lang/crates.io-index)",
+                ]
 
-            [[package]]
-            name = "chrono"
-            version = "0.2.0"
-            source = "registry+https://github.com/rust-lang/crates.io-index"
-            replace = "chrono 0.2.0"
+                [[package]]
+                name = "chrono"
+                version = "0.2.0"
+                source = "registry+https://github.com/rust-lang/crates.io-index"
+                replace = "chrono 0.2.0"
 
-            [[package]]
-            name = "chrono"
-            version = "0.2.0"
-            dependencies = [
-             "serde 0.7.0 (registry+https://github.com/rust-lang/crates.io-index)",
-            ]
+                [[package]]
+                name = "chrono"
+                version = "0.2.0"
+                dependencies = [
+                 "serde 0.7.0 (registry+https://github.com/rust-lang/crates.io-index)",
+                ]
 
-            [[package]]
-            name = "serde"
-            version = "0.7.0"
-            source = "registry+https://github.com/rust-lang/crates.io-index"
+                [[package]]
+                name = "serde"
+                version = "0.7.0"
+                source = "registry+https://github.com/rust-lang/crates.io-index"
 
-            [[package]]
-            name = "serde"
-            version = "0.8.0"
-            source = "registry+https://github.com/rust-lang/crates.io-index"
-            replace = "serde 0.8.0"
+                [[package]]
+                name = "serde"
+                version = "0.8.0"
+                source = "registry+https://github.com/rust-lang/crates.io-index"
+                replace = "serde 0.8.0"
 
-            [[package]]
-            name = "serde"
-            version = "0.8.0"
-        "#,
+                [[package]]
+                name = "serde"
+                version = "0.8.0"
+            "#,
         )
         .file(
             "src/lib.rs",
@@ -900,14 +1007,14 @@ fn override_an_override() {
         .file(
             "chrono/Cargo.toml",
             r#"
-            [package]
-            name = "chrono"
-            version = "0.2.0"
-            authors = []
+                [package]
+                name = "chrono"
+                version = "0.2.0"
+                authors = []
 
-            [dependencies]
-            serde = "< 0.9"
-        "#,
+                [dependencies]
+                serde = "< 0.9"
+            "#,
         )
         .file(
             "chrono/src/lib.rs",
@@ -922,7 +1029,7 @@ fn override_an_override() {
         .file("serde/src/lib.rs", "pub fn serde08_override() {}")
         .build();
 
-    p.cargo("build -v").run();
+    p.cargo("check -v").run();
 }
 
 #[cargo_test]
@@ -934,14 +1041,14 @@ fn overriding_nonexistent_no_spurious() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
+                [package]
+                name = "bar"
+                version = "0.1.0"
+                authors = []
 
-            [dependencies]
-            baz = { path = "baz" }
-        "#,
+                [dependencies]
+                baz = { path = "baz" }
+            "#,
         )
         .file("src/lib.rs", "pub fn bar() {}")
         .file("baz/Cargo.toml", &basic_manifest("baz", "0.1.0"))
@@ -953,29 +1060,29 @@ fn overriding_nonexistent_no_spurious() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
 
-            [dependencies]
-            bar = "0.1.0"
+                    [dependencies]
+                    bar = "0.1.0"
 
-            [replace]
-            "bar:0.1.0" = {{ git = '{url}' }}
-            "baz:0.1.0" = {{ git = '{url}' }}
-        "#,
+                    [replace]
+                    "bar:0.1.0" = {{ git = '{url}' }}
+                    "baz:0.1.0" = {{ git = '{url}' }}
+                "#,
                 url = bar.url()
             ),
         )
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build").run();
-    p.cargo("build")
+    p.cargo("check").run();
+    p.cargo("check")
         .with_stderr(
             "\
-[WARNING] package replacement is not used: [..]baz:0.1.0
+[WARNING] package replacement is not used: [..]baz@0.1.0
 [FINISHED] [..]
 ",
         )
@@ -996,18 +1103,19 @@ fn no_warnings_when_replace_is_used_in_another_workspace_member() {
             members = [ "first_crate", "second_crate"]
 
             [replace]
-            "bar:0.1.0" = { path = "local_bar" }"#,
+            "bar:0.1.0" = { path = "local_bar" }
+            "#,
         )
         .file(
             "first_crate/Cargo.toml",
             r#"
-            [package]
-            name = "first_crate"
-            version = "0.1.0"
+                [package]
+                name = "first_crate"
+                version = "0.1.0"
 
-            [dependencies]
-            bar = "0.1.0"
-        "#,
+                [dependencies]
+                bar = "0.1.0"
+            "#,
         )
         .file("first_crate/src/lib.rs", "")
         .file(
@@ -1019,24 +1127,24 @@ fn no_warnings_when_replace_is_used_in_another_workspace_member() {
         .file("local_bar/src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .cwd("first_crate")
         .with_stdout("")
         .with_stderr(
             "\
 [UPDATING] `[..]` index
-[COMPILING] bar v0.1.0 ([..])
-[COMPILING] first_crate v0.1.0 ([..])
+[CHECKING] bar v0.1.0 ([..])
+[CHECKING] first_crate v0.1.0 ([..])
 [FINISHED] [..]",
         )
         .run();
 
-    p.cargo("build")
+    p.cargo("check")
         .cwd("second_crate")
         .with_stdout("")
         .with_stderr(
             "\
-[COMPILING] second_crate v0.1.0 ([..])
+[CHECKING] second_crate v0.1.0 ([..])
 [FINISHED] [..]",
         )
         .run();
@@ -1051,30 +1159,30 @@ fn replace_to_path_dep() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies]
-            bar = "0.1.0"
+                [dependencies]
+                bar = "0.1.0"
 
-            [replace]
-            "bar:0.1.0" = { path = "bar" }
-        "#,
+                [replace]
+                "bar:0.1.0" = { path = "bar" }
+            "#,
         )
         .file("src/lib.rs", "extern crate bar;")
         .file(
             "bar/Cargo.toml",
             r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
+                [package]
+                name = "bar"
+                version = "0.1.0"
+                authors = []
 
-            [dependencies]
-            baz = { path = "baz" }
-        "#,
+                [dependencies]
+                baz = { path = "baz" }
+            "#,
         )
         .file(
             "bar/src/lib.rs",
@@ -1084,7 +1192,7 @@ fn replace_to_path_dep() {
         .file("bar/baz/src/lib.rs", "pub fn baz() {}")
         .build();
 
-    p.cargo("build").run();
+    p.cargo("check").run();
 }
 
 #[cargo_test]
@@ -1097,51 +1205,51 @@ fn override_with_default_feature() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies]
-            bar = { path = "bar", default-features = false }
-            another = "0.1"
-            another2 = { path = "another2" }
+                [dependencies]
+                bar = { path = "bar", default-features = false }
+                another = "0.1"
+                another2 = { path = "another2" }
 
-            [replace]
-            'bar:0.1.0' = { path = "bar" }
-        "#,
+                [replace]
+                'bar:0.1.0' = { path = "bar" }
+            "#,
         )
         .file("src/main.rs", "extern crate bar; fn main() { bar::bar(); }")
         .file(
             "bar/Cargo.toml",
             r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
+                [package]
+                name = "bar"
+                version = "0.1.0"
+                authors = []
 
-            [features]
-            default = []
-        "#,
+                [features]
+                default = []
+            "#,
         )
         .file(
             "bar/src/lib.rs",
             r#"
-            #[cfg(feature = "default")]
-            pub fn bar() {}
-        "#,
+                #[cfg(feature = "default")]
+                pub fn bar() {}
+            "#,
         )
         .file(
             "another2/Cargo.toml",
             r#"
-            [package]
-            name = "another2"
-            version = "0.1.0"
-            authors = []
+                [package]
+                name = "another2"
+                version = "0.1.0"
+                authors = []
 
-            [dependencies]
-            bar = { version = "0.1", default-features = false }
-        "#,
+                [dependencies]
+                bar = { version = "0.1", default-features = false }
+            "#,
         )
         .file("another2/src/lib.rs", "")
         .build();
@@ -1157,35 +1265,35 @@ fn override_plus_dep() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies]
-            bar = "0.1"
+                [dependencies]
+                bar = "0.1"
 
-            [replace]
-            'bar:0.1.0' = { path = "bar" }
-        "#,
+                [replace]
+                'bar:0.1.0' = { path = "bar" }
+            "#,
         )
         .file("src/lib.rs", "")
         .file(
             "bar/Cargo.toml",
             r#"
-            [package]
-            name = "bar"
-            version = "0.1.0"
-            authors = []
+                [package]
+                name = "bar"
+                version = "0.1.0"
+                authors = []
 
-            [dependencies]
-            foo = { path = ".." }
-        "#,
+                [dependencies]
+                foo = { path = ".." }
+            "#,
         )
         .file("bar/src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr_contains("error: cyclic package dependency: [..]")
         .run();
